<?php
/**
 * @author Jaap Jansma <jaap.jansma@civicoop.org>
 * @license AGPL-3.0
 */

use Civi\DataProcessor\FilterHandler\AbstractFilterHandler;
use Civi\DataProcessor\Output\UIOutputInterface;
use Civi\DataProcessor\ProcessorType\AbstractProcessorType;

abstract class CRM_Dataprocessor_Form_Output_AbstractUIOutputForm extends CRM_Core_Form_Search {

  /**
   * @var array
   */
  protected $dataProcessor;

  /**
   * @var \Civi\DataProcessor\ProcessorType\AbstractProcessorType;
   */
  protected $dataProcessorClass;

  /**
   * @var int
   */
  protected $dataProcessorId;

  /**
   * @var \CRM_Dataprocessor_BAO_DataProcessorOutput
   */
  protected $dataProcessorOutput;

  /**
   * Return the data processor ID
   *
   * @return String
   */
  abstract protected function getDataProcessorName(): string;

  /**
   * Returns the name of the output for this search
   *
   * @return string
   */
  abstract protected function getOutputName(): string;

  /**
   * Checks whether the output has a valid configuration
   *
   * @return bool
   */
  abstract protected function isConfigurationValid(): bool;

  public function preProcess() {
    parent::preProcess();
    try {
      $this->loadDataProcessor();
    } catch (Exception $e) {
      // Do nothing.
    }
    $this->assign('has_exposed_filters', $this->hasExposedFilters());
  }

  public function buildQuickForm() {
    parent::buildQuickForm();
    $this->add('hidden', 'debug');
    $this->setDefaults(['debug' => $this->isDebug()]);
  }

  /**
   * @return \Civi\DataProcessor\ProcessorType\AbstractProcessorType
   */
  public function getDataProcessorClass(): AbstractProcessorType {
    return $this->dataProcessorClass;
  }

  /**
   * @return array
   */
  public function getDataProcessor(): array {
    return $this->dataProcessor;
  }

  protected function isDebug(): bool {
    try {
      $debug = CRM_Utils_Request::retrieve('debug', 'Boolean');
    } catch (CRM_Core_Exception $e) {
      return false;
    }
    if (!$debug) {
      $debug = $this->_formValues['debug'] ?? FALSE;
    }
    return (bool) $debug;
  }

  /**
   * Retrieve the data processor and the output configuration
   *
   * @throws \Exception
   */
  protected function loadDataProcessor() {
    $factory = dataprocessor_get_factory();
    if (!$this->dataProcessorId) {
      $doNotUseCache = $this->isDebug();

      $dataProcessorName = $this->getDataProcessorName();
      $sql = "
        SELECT civicrm_data_processor.id as data_processor_id,  civicrm_data_processor_output.id AS output_id
        FROM civicrm_data_processor
        INNER JOIN civicrm_data_processor_output ON civicrm_data_processor.id = civicrm_data_processor_output.data_processor_id
        WHERE is_active = 1 AND civicrm_data_processor.name = %1 AND civicrm_data_processor_output.type = %2
      ";
      $params[1] = [$dataProcessorName, 'String'];
      $params[2] = [$this->getOutputName(), 'String'];
      $dao = CRM_Dataprocessor_BAO_DataProcessor::executeQuery($sql, $params, TRUE, 'CRM_Dataprocessor_BAO_DataProcessor');
      if (!$dao->fetch()) {
        throw new Exception('Could not find Data Processor "' . $dataProcessorName.'"');
      }

      $this->dataProcessor = civicrm_api3('DataProcessor', 'getsingle', array('id' => $dao->data_processor_id));
      $this->dataProcessorClass = CRM_Dataprocessor_BAO_DataProcessor::dataProcessorToClass($this->dataProcessor, $doNotUseCache);
      $this->dataProcessorId = $dao->data_processor_id;

      $this->dataProcessorOutput = civicrm_api3('DataProcessorOutput', 'getsingle', array('id' => $dao->output_id));
      $this->dataProcessorOutput = $this->alterDataProcessorOutput($this->dataProcessorOutput);
      $this->assign('output', $this->dataProcessorOutput);

      $outputClass = $factory->getOutputByName($this->dataProcessorOutput['type']);
      if (!$outputClass instanceof UIOutputInterface) {
        throw new Exception('Invalid output');
      }

      if (!$outputClass->checkUIPermission($this->dataProcessorOutput, $this->dataProcessor)) {
        CRM_Utils_System::permissionDenied();
        CRM_Utils_System::civiExit();
      } elseif (!$this->isConfigurationValid()) {
        throw new Exception('Invalid configuration found of the data processor "' . $dataProcessorName . '"');
      }
    }
  }

  /**
   * This function could be override in child classes to change default configuration.
   *
   * @param $output
   *
   * @return array
   */
  protected function alterDataProcessorOutput($output): array {
    return $output;
  }

  /**
   * Returns whether the search has required filters.
   *
   * @return bool
   */
  protected function hasRequiredFilters(): bool {
    if ($this->dataProcessorClass->getFilterHandlers()) {
      foreach ($this->dataProcessorClass->getFilterHandlers() as $filter) {
        if ($filter->isRequired() && $filter->isExposed()) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Returns whether the search has required filters.
   *
   * @return bool
   */
  protected function hasExposedFilters(): bool {
    if ($this->dataProcessorClass->getFilterHandlers()) {
      foreach ($this->dataProcessorClass->getFilterHandlers() as $filter) {
        if ($filter->isExposed()) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Validate the filters
   *
   * @return array
   */
  protected function validateFilters(): array {
    $errors = array();
    if ($this->dataProcessorClass->getFilterHandlers()) {
      foreach ($this->dataProcessorClass->getFilterHandlers() as $filter) {
        if ($filter->isExposed()) {
          $errors = array_merge($errors, $filter->validateSubmittedFilterParams($this->_formValues));
        }
      }
    }
    return $errors;
  }

  /**
   * Apply the filters to the database processor
   */
  public static function applyFilters(AbstractProcessorType $dataProcessor, $submittedValues) {
    if ($dataProcessor->getFilterHandlers()) {
      foreach ($dataProcessor->getFilterHandlers() as $filter) {
        if ($filter->isExposed()) {
          $filter->resetFilter();
          $filterValues = $filter->processSubmittedValues($submittedValues);
          if (empty($filterValues)) {
            $filterValues = self::getDefaultFilterValues($filter);
          }
          try {
            $filter->applyFilterFromSubmittedFilterParams($filterValues);
          } catch (Exception $e) {
            // Do nothing.
          }
        }
      }
    }
  }

  /**
   * Get the default filter values for a filter. If there is no default value we allow the value to be set by a URL parameter of the same name as the filter.
   *
   * @param \Civi\DataProcessor\FilterHandler\AbstractFilterHandler $filterHandler
   *
   * @return array
   */
  public static function getDefaultFilterValues(AbstractFilterHandler $filterHandler): array {
    $defaultOp = $filterHandler->getDefaultOperator();
    $type = ($filterHandler->getFieldSpecification()->type === 'Int') ? 'CommaSeparatedIntegers' : $filterHandler->getFieldSpecification()->type;
    $filterValues = [];

    try {
      $valueFromURL = CRM_Utils_Request::retrieveValue($filterHandler->getFieldSpecification()->alias, $type, NULL, FALSE, 'GET');
      if ($valueFromURL) {
        $filterValues = [
          'op' => $defaultOp,
          'value' => $valueFromURL,
        ];
      }
    } catch (CRM_Core_Exception $e) {
      // Do nothing.
    }
    if (empty($filterValues)) {
      $filterValues = $filterHandler->getDefaultFilterValues();
    }
    return $filterValues;
  }

  /**
   * Build the criteria form
   */
  protected function buildCriteriaForm() {
    $filterElements = array();
    if ($this->dataProcessorClass->getFilterHandlers()) {
      foreach ($this->dataProcessorClass->getFilterHandlers() as $filterHandler) {
        $fieldSpec = $filterHandler->getFieldSpecification();
        if (!$fieldSpec || !$filterHandler->isExposed()) {
          continue;
        }
        $filterElements[$fieldSpec->alias]['filter'] = $filterHandler->addToFilterForm($this, self::getDefaultFilterValues($filterHandler), $this->getCriteriaElementSize());
        $filterElements[$fieldSpec->alias]['template'] = $filterHandler->getTemplateFileName();
      }
      $this->assign('filters', $filterElements);
    }
    $this->assign('additional_criteria_template', $this->getAdditionalCriteriaTemplate());
  }

  /**
   * Returns the size of the criteria form element.
   * There are two sizes full and compact.
   *
   * @return string
   */
  protected function getCriteriaElementSize(): string {
    return 'full';
  }

  /**
   * Returns the name of the additional criteria template.
   *
   * @return false|String|array
   */
  protected function getAdditionalCriteriaTemplate():? string {
    return false;
  }
}
